#
# Copyright (c) 2002 Gilles Lenfant
# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
'''
Base classes for the XQuizz product

Resources have been included in this module to have the maximum of
testable features without the Zope framework

This module exposes the Quizz, Question and Answer classes

$Id: quizz.py,v 1.1.1.1 2002/05/27 02:22:35 glt Exp $
'''

__version__ = "$Revision: 1.1.1.1 $"[11:-2]

__all__ = ['Quizz', 'Question', 'Answer']

from UserList import UserList

###############
## Utilities ##
###############

def id2index(someid):
    '''Index of an object in its parent
    '''
    return int(someid[-3:])

def questionId(index):
    '''Makes an ID for a Question
    '''
    return 'q_%03d' % index

def answerId(index):
    '''Makes an ID for an Answer
    '''
    return 'a_%03d' % index

##########################
## Product core classes ##
##########################

class Quizz(UserList):
    '''Basic Quizz without Zope features
    '''
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, initquestions=[]):
        '''Constructor.
        initquestions = [] : initial set of question objects
        '''
        for question in initquestions:
            question.parent = self
        UserList.__init__(self, initquestions)
        return

    def maximumScore(self):
        '''Highest possible score of the Quizz object
        -> integer
        '''
        return reduce(lambda x, y: x + y.maximumScore(), self, 0)

    def addQuestion(self, question):
        '''Appends a question to the Quizz object
        question is a Question object
        '''
        question.parent = self
        self.append(question)
        return

    def getScore(self, replies):
        '''Score of an user for this Quizz object
        -> integer
        replies is a mapping object that contains what the user replied
        '''
        # List of possible keys for the quizz
        qnames = map(lambda x: questionId(x), range(len(self)))
        score = 0
        for k, v in replies.items():
            # Filter other keys (Web form...)
            if k in qnames:
                question = self[id2index(k)]
                answer = question[id2index(v)]
                score = score + answer.points
        return score
# /class Quizz
    
class Question(UserList):
    '''One question in the Quizz
    '''
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, parent=None, label='', initanswers=[]):
        '''Constructor.
        parent = None : the Quizz container object
        label = '' : the text for this question
        initanswers = [] : possible answers for this Question
        '''
        self.parent = parent
        self.label = label
        for answer in initanswers:
            answer.parent = self
        UserList.__init__(self, initanswers)
        return

    def maximumScore(self):
        '''Best score for this question
        -> integer
        '''
        return reduce(lambda x,y : max(x, y.points), self, 0)

    def addAnswer(self, answer):
        '''Appends a Answer object to this Question object
        '''
        answer.parent = self
        self.append(answer)
        return

    def bestAnswer(self):
        '''The label of the best answer
        -> string
        '''
        best = None
        for answer in self:
            try:
                if answer.points > best.points:
                    best = answer
            except AttributeError:
                # When "best" is None
                best = answer
        return best.label

    def getId(self):
        '''Makes an id for this Question
        -> string if success or None
        '''
        xid = 0
        for aquestion in self.parent:
            if aquestion is self:
                return questionId(xid)
            xid = xid + 1
        return None
# /class Question

class Answer:
    __allow_access_to_unprotected_subobjects__ = 1
    def __init__(self, parent=None, label='', points=0):
        '''Constructor.
        parent = None : the Question container object
        label = '' : the text for this Answer
        points = 0 : value for this Answer object
        '''
        self.parent = parent
        self.label = label
        self.points = points
        return

    def getId(self):
        '''Makes an id for this Answer
        -> string if success or None
        '''
        xid = 0
        for ananswer in self.parent:
            if ananswer is self:
                return answerId(xid)
            xid = xid + 1
        return None

    def widget(self):
        '''Renders an HTML widget for this Answer
        -> HTML string
        This requires this Answer is in a Question that is in a Quizz
        '''
        template = '<input type="radio" name="%s" value="%s">'
        return template % (self.parent.getId(), self.getId())
# /class Answer